/*
 *This file is part of the Configuration File Library (CFL).
 * Copyright (C) 2002 Nicolas Darnis <ndarnis@free.fr>
 *
 * The CFL library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The CFL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the CFL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: cfl.h
 * $Revision: 1.5 $
 * $Date: 2002/11/06 11:45:23 $
 * $Author: ndarnis $
 * $Id: cfl.h,v 1.5 2002/11/06 11:45:23 ndarnis Exp $
 */

#ifndef _CFL_H_
#define _CFL_H_

#include <stdio.h>

#if defined (__cplusplus)
extern "C" {
#endif /* __cplusplus */

/**
 * cfl_value_type_t
 *
 * This is the list of available CFL values types
 */
typedef enum 
{ 
  /** Boolean type */
  CFL_TYPE_BOOL, 
  /** NULL terminated characters string type */
  CFL_TYPE_STRING, 
  /** Integer type (long) */
  CFL_TYPE_INTEGER, 
  /** Float type (double) */
  CFL_TYPE_FLOAT 
} cfl_value_type_t;

/**
 * The CFL handle type
 */
typedef struct _cfl_handle *cflh_t;

/**
 * Allocates a new cfl handle.
 * Creates and allocate a new cfl handle.
 * Returns the newly allocated handle in case of success.
 * Returns NULL in case of failure.
 *
 * Requires: nothing
 *
 * @see cfl_read()
 * @see cfl_free()
 */
extern cflh_t
cfl_alloc (void);

/**
 * Deallocate a cfl handle.
 * Deallocates and destroy the cfl HANDLE previously created by cfl_alloc() or 
 * cfl_read().
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle to deallocate
 * @see cfl_alloc()
 * @see cfl_read()
 */
extern void
cfl_free (cflh_t HANDLE                           /* The handle to deallocate */
	  );

/**
 * Creates a cfl handle from a file.
 * Open and the file INPUT_FILE_NAME to create a new cfl handle. The file is
 * closed after a call to this function. Read the file grammar.txt to in the
 * doc/ directory of the distribution to learn which kind of files this function 
 * could read. 
 * Returns the newly allocated handle in case of success.
 * Returns NULL in case of failure.
 *
 * WARNING: this function is *NOT* reentrant, so you should lock a mutex before
 *          calling it and unlock the mutex after the call.
 *
 * Requires: nothing
 *
 * @param INPUT_FILE_NAME The name of the configuration file to read
 * @see cfl_alloc()
 * @see cfl_free()
 * @see cfl_write()
 * @see cfl_write_xml()
 */
extern cflh_t
cfl_read (const char *INPUT_FILE_NAME     /* The file to read the config from */
	  );

/**
 * Write a cfl configuration handle to a file.
 * Writes the configuration contained in the cfl HANDLE to the file OUTPUT_FILE.
 * Returns 0 in case of success.
 * Returns -1 in case of failure.
 *
 * Requires: HANDLE must be a valid cfl handle
 *           & OUTPUT_FILE must be a valid file
 *
 * @param HANDLE The configuration cfl handle to write
 * @param OUTPUT_FILE The file to write the configuration into
 * @see cfl_read()
 * @see cfl_write_xml()
 */
extern int
cfl_write (const cflh_t HANDLE,                    /* The CFL handle to write */
	   FILE *OUTPUT_FILE                 /* The output file to write into */
	   );

/**
 * Writes a cfl configuration handle to a file into XML.
 * Writes the configuration contained in the cfl HANDLE to the file OUTPUT_FILE
 * into XML language.
 * Returns 0 in case of success.
 * Returns -1 in case of failure.
 *
 * Requires: HANDLE must be a valid cfl handle
 *           & OUTPUT_FILE must be a valid file
 *
 * @param HANDLE The configuration cfl handle to write
 * @param OUTPUT_FILE The file to write the configuration into
 * @see cfl_read()
 * @see cfl_write()
 */
extern int
cfl_write_xml (const cflh_t HANDLE,                    /* CFL handle to write */
	       FILE *OUTPUT_FILE                           /* The output file */
	       );

/**
 * Adds a new section into a cfl handle.
 * Adds a new section named SECTION_NAME into the cfl HANDLE.
 * Returns 0 in case of success.
 * Returns -1 if a section named SECTION_NAME already exists.
 * Returns -2 in other cases of failure.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle in which the new section must go
 * @param SECTION_NAME The name of the new section to add
 * @see cfl_section_remove()
 */
extern int
cfl_section_insert (cflh_t HANDLE,     /* CFL handle to insert the section in */
		    const char *SECTION_NAME /* Name of the section to insert */
		    );

/**
 * Alias for cfl_section_insert()
 * @see cfl_section_insert()
 */
#define cfl_section_add (h,sn) cfl_section_insert (h, sn);

/**
 * Removes a section from a cfl handle.
 * Removes the section named SECTION_NAME from the cfl HANDLE.
 * Returns 0 in case of success.
 * Returns -1 if no section SECTION_NAME exists into the HANDLE.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle to remove the section from
 * @param SECTION_NAME The name of the section to remove
 * @see cfl_section_add()
 */
extern int
cfl_section_remove (cflh_t HANDLE,   /* CFL handle to remove the section from */
		    const char *SECTION_NAME /* Name of the section to remove */
		    );

/**
 * Gets the number of sections in a cfl handle.
 * Returns the number of sections contained in the cfl HANDLE.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle to ciunt sections from
 * @see cfl_section_get_name_by_position()
 */
extern int
cfl_get_sections_count (cflh_t HANDLE    /* CFL handle to count sections from */
			);

/**
 * Gets the name of the section in a given position from a cfl handle.
 * Returns the section's name at the position POS of the cfl HANDLE.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle from which geting a section name
 * @param POS The position of the section to get the name
 * @see cfl_get_sections_count()
 */
extern char *
cfl_section_get_name_by_position (cflh_t HANDLE, 
				  unsigned long POS
				  );

/**
 * Gets the number of variables from a section.
 * Returns the number of variables into the section SECTION_NAME of the CFG 
 * config data structure.
 * Returns the number of variables in case of success.
 * Returns -1 in case of failure.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle to use
 * @param SECTION_NAME The name of the section to get the number of values from
 * @see cfl_section_add()
 * @see cfl_section_remove()
 */
extern long
cfl_section_get_values_count (const cflh_t HANDLE,   /* The cfl handle to use */
			      const char *SECTION_NAME        /* Section name */
			      );

/**
 * Alias for cfl_section_get_values_count()
 * @see cfl_section_get_values_count()
 */
#define cfl_section_get_values_number(h,sn) cfl_section_get_values_count(h, sn)

/**
 * Add a new value into a section of a cfl handle.
 * Add the new value named VALUE_NAME in the section SECTION_NAME into the cfl
 * HANDLE. The new value will be of type VALUE_TYPE and of value VALUE.
 * Returns 0 in case of success.
 * Returns -1 if no section SECTION_NAME is found.
 * Returns -2 if a value named VALUE_NAME already exists.
 * Returns -3 in other cases of failure.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle in which the new value must go
 * @param SECTION_NAME The name of the section in which the new value must go
 * @param VALUE_NAME The name of the value to insert
 * @param VALUE_TYPE The type of the value to insert
 * @param VALUE The value of the value to insert
 * @see cfl_value_remove()
 */
extern int
cfl_value_insert (cflh_t HANDLE,         /* CFL handle to insert a value into */
		  const char *SECTION_NAME,     /* Section to insert value in */
		  const char *VALUE_NAME,      /* Name of the value to insert */
		  const cfl_value_type_t VALUE_TYPE,            /* Value type */
		  void *VALUE                             /* The value itself */
		  );

/**
 * Alias for cfl_value_insert()
 * @see cfl_value_insert()
 */
#define cfl_value_add (h,sn,vn,vt,vl) cfl_value_insert (h, sn, vn, vt, vl)

/**
 * Remove a value from a section from a cfl handle.
 * Removes the value VALUE_NAME from the section SECTION_NAME of the cfl HANDLE.
 * Returns 0 in case of success.
 * Returns -1 if no section SECTION_NAME is found.
 * Returns -2 if no value VALUE_NAME is found.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle to remove the value from
 * @param SECTION_NAME The name of the section in which the value is
 * @param VALUE_NAME The name of the value to remove
 * @see cfl_value_add()
 */
extern int
cfl_value_remove (cflh_t HANDLE,       /* CFL handle to remove the value from */
		  const char *SECTION_NAME,     /* Section where the value is */
		  const char *VALUE_NAME       /* Name of the value to remove */
		  );

/**
 * Modifies a value of a cfl handle.
 * Changes the value VALUE_NAME of the section SECTION_NAME into the cfl HANDLE
 * with type VALUE_TYPE and value VALUE.
 * Returns 0 in case of success.
 * Returns -1 if no section SECTION_NAME is found.
 * Returns -2 if no value VALUE_NAME is found.
 * Returns -3 in other cases of failure.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle in which the value must be modified
 * @param SECTION_NAME The section in which the value is
 * @param VALUE_NAME The name of the value to modify
 * @param VALUE_TYPE The new type of the value
 * @param VALUE The new value of the value
 * @see cfl_value_get_by_name()
 * @see cfl_value_get_by_position()
 */
extern int
cfl_value_set_by_name (cflh_t HANLDE,    /* CFL handle to insert a value into */
		       const char *S_NAME,      /* Section to insert value in */
		       const char *V_NAME,     /* Name of the value to insert */
		       const cfl_value_type_t V_TYPE,           /* Value type */
		       void *VALUE                        /* The value itself */
		       );

/**
 * Gets a value by its name from a section in cfl handle.
 * Returns the address of the value V_NAME from the section S_NAME in case of 
 * success. The type of the found value is set to V_TYPE.
 * Returns NULL in case of failure.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle from which getting the value
 * @param SECTION_NAME The name of the section in which the value is
 * @param VALUE_NAME The name of the value to get
 * @param VALUE_TYPE The address in which the type of the value will go
 * @see cfl_value_set_by_name()
 * @see cfl_value_get_by_position()
 */
extern void *
cfl_value_get_by_name (cflh_t HANDLE,     /* CFL handle to get the value from */
		       const char *SECTION_NAME,/* Section where the value is */
		       const char *VALUE_NAME,    /* Name of the value to get */
		       cfl_value_type_t *VALUE_TYPE             /* Value type */
		       );

/**
 * Gets a value by its position from a cfl handle.
 * Returns the address of the value at position POS from the section 
 * SECTION_NAME in the cfl HANDLE in case of success. The name of the founded 
 * value is set to VALUE_NAME and its type to VALUE_TYPE.
 * Returns NULL in case of failure.
 *
 * Requires: HANDLE must be a valid cfl handle
 *
 * @param HANDLE The cfl handle from which getting the value
 * @param SECTION_NAME The name of the section in which the value is
 * @param POS The position of the value to get
 * @param VALUE_NAME The address in which the name of the value will go
 * @param VALUE_TYPE The address in which the type of the value will go
 * @see cfl_value_get_by_name()
 * @see cfl_value_set_by_name()
 */
extern void *
cfl_value_get_by_position (const cflh_t HANDLE,   /* Handle to get value from */
			   const char *SECTION_NAME,  /* Section of the value */
			   unsigned long POS, /* Position of the value to get */
			   char **VALUE_NAME,                   /* Value name */
			   cfl_value_type_t *VALUE_TYPE         /* Value type */
			   );
#ifdef __cplusplus
}
#endif/* __cplusplus */

#endif /* _CFL_H_ */
